package org.xml.sax.helpers;

import java.io.InputStream;
import org.xml.sax.SAXException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import org.xml.sax.XMLReader;

public final class XMLReaderFactory
{
    private static final String property = "org.xml.sax.driver";
    private static SecuritySupport ss;
    private static String _clsFromJar;
    private static boolean _jarread;
    
    private XMLReaderFactory() {
    }
    
    public static XMLReader createXMLReader() throws SAXException {
        String className = null;
        ClassLoader cl = XMLReaderFactory.ss.getContextClassLoader();
        try {
            className = XMLReaderFactory.ss.getSystemProperty("org.xml.sax.driver");
        }
        catch (RuntimeException ex) {}
        if (className == null) {
            if (!XMLReaderFactory._jarread) {
                XMLReaderFactory._jarread = true;
                String service = "META-INF/services/org.xml.sax.driver";
                try {
                    InputStream in;
                    if (cl != null) {
                        in = XMLReaderFactory.ss.getResourceAsStream(cl, service);
                        if (in == null) {
                            cl = null;
                            in = XMLReaderFactory.ss.getResourceAsStream(cl, service);
                        }
                    }
                    else {
                        in = XMLReaderFactory.ss.getResourceAsStream(cl, service);
                    }
                    if (in != null) {
                        BufferedReader reader = new BufferedReader(new InputStreamReader(in, "UTF8"));
                        XMLReaderFactory._clsFromJar = reader.readLine();
                        in.close();
                    }
                }
                catch (Exception ex2) {}
            }
            className = XMLReaderFactory._clsFromJar;
        }
        if (className == null) {
            className = "com.sun.org.apache.xerces.internal.parsers.SAXParser";
        }
        if (className != null) {
            return loadClass(cl, className);
        }
        try {
            return new ParserAdapter(ParserFactory.makeParser());
        }
        catch (Exception e) {
            throw new SAXException("Can't create default XMLReader; is system property org.xml.sax.driver set?");
        }
    }
    
    public static XMLReader createXMLReader(String className) throws SAXException {
        return loadClass(XMLReaderFactory.ss.getContextClassLoader(), className);
    }
    
    private static XMLReader loadClass(ClassLoader loader, String className) throws SAXException {
        try {
            return (XMLReader)NewInstance.newInstance(loader, className);
        }
        catch (ClassNotFoundException e1) {
            throw new SAXException("SAX2 driver class " + className + " not found", e1);
        }
        catch (IllegalAccessException e2) {
            throw new SAXException("SAX2 driver class " + className + " found but cannot be loaded", e2);
        }
        catch (InstantiationException e3) {
            throw new SAXException("SAX2 driver class " + className + " loaded but cannot be instantiated (no empty public constructor?)", e3);
        }
        catch (ClassCastException e4) {
            throw new SAXException("SAX2 driver class " + className + " does not implement XMLReader", e4);
        }
    }
    
    static {
        XMLReaderFactory.ss = new SecuritySupport();
        XMLReaderFactory._clsFromJar = null;
        XMLReaderFactory._jarread = false;
    }
}
